//Title: Skeleton Main Window Class
//Platform: BeOS 5
//Version: 2001.12.18
//Description:
//	A class that creates the application's "main" window.
//	This class inherits from BWindow, and runs a message loop for the window.

//Copyright (C) 2001, 2002 Kevin H. Patterson


#include "clsMainWindow.h"

clsMainWindow::clsMainWindow(const char *uWindowTitle)
:
	BWindow(
		BRect(64, 64, 320, 256),
		uWindowTitle,
		B_TITLED_WINDOW,
		0	)
{
	SelectedDevice = -1;
	SelectedMode = -1;

	BView* MainView(
		new BView(BWindow::Bounds(), NULL, B_FOLLOW_ALL, 0) );
	if (MainView != NULL)
	{
		MainView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
		BWindow::AddChild(MainView);
	}
	else
	{
		AppReturnValue = B_NO_MEMORY;
		be_app->PostMessage(B_QUIT_REQUESTED);
	}
	
	BPopUpMenu *DeviceItems(new BPopUpMenu("Device items"));
	if (DeviceItems != NULL)
	{
		Device = new BMenuField(
				BRect(7.0f, 5.0f, MainView->Bounds().right - 27.0f, 25.0f),
				"Device",
				"Device:",
				DeviceItems,
				true,
				B_FOLLOW_RIGHT | B_FOLLOW_TOP );
		if (Device != NULL)
		{
			Device->SetDivider(Device->Bounds().right - 88.0f);
			Device->SetAlignment(B_ALIGN_RIGHT);
			MainView->AddChild(Device);

			DeviceManager * tDeviceManager(new DeviceManager);
			if (tDeviceManager != NULL) {

				BMenuItem *tMenuItem(NULL);
				BString DeviceName;
				uint32 DeviceNumber(1);
				
				GraphicsDevice * tGraphicsDevice(NULL);
				uint32 GraphicsDevices = tDeviceManager->GraphicsDeviceCount();
				for (uint32 idx(0); idx < GraphicsDevices; ++idx) {
					tGraphicsDevice = tDeviceManager->GetGraphicsDevice(idx);
					if (tGraphicsDevice != NULL) {
						DeviceName = BString("");
						DeviceName << DeviceNumber << " - ";
						DeviceName << tGraphicsDevice->Name();
						tMenuItem = new BMenuItem( DeviceName.String(), new BMessage('DevA' + idx) );
						Device->Menu()->AddItem(tMenuItem);
						DeviceNumber++;
					}
					delete tGraphicsDevice;
					tGraphicsDevice = NULL;
				}
				delete tDeviceManager;
				tDeviceManager = NULL;
			}

		}
	}
	
	BPopUpMenu *ModeItems(new BPopUpMenu("Mode items"));
	if (ModeItems != NULL)
	{
		Mode = new BMenuField(
				BRect(7.0f, 30.0f, MainView->Bounds().right - 27.0f, 50.0f),
				"Mode",
				"Mode:",
				ModeItems,
				true,
				B_FOLLOW_RIGHT | B_FOLLOW_TOP );
		if (Mode != NULL)
		{
			Mode->SetDivider(Mode->Bounds().right - 88.0f);
			Mode->SetAlignment(B_ALIGN_RIGHT);
			MainView->AddChild(Mode);

			BMenuItem *tMenuItem(NULL);
			uint32 ModeNumber(1);
			
			char * ModeName(NULL);
			ModeName = "1 - 640x480 NTSC testimage";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
			ModeName = "2 - 640x480 NTSC TV output";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
			ModeName = "3 - 800x600 NTSC TV output";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
			ModeName = "4 - 800x600 PAL testimage";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
			ModeName = "5 - 640x480 PAL TV output";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
			ModeName = "6 - 800x600 PAL TV output";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
			ModeName = "7 - VGA Only (broken?)";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
			ModeName = "8 - Exit VGA Only (broken?)";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
			ModeName = "9 - 720x480 overscanning NTSC TV output (NTSC DVD/MPEG2 mode)";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
			ModeName = "10 - 720x576 overscanning PAL TV output (PAL DVD/MPEG2 mode)";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
			ModeName = "11 - 640x480 overscanning NTSC TV output (NTSC VCD/MPEG1 mode)";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
			ModeName = "12 - 768x576 overscanning PAL TV output (PAL VCD/MPEG1 mode)";
				tMenuItem = new BMenuItem( ModeName, new BMessage('ModA' + ModeNumber) );
				Mode->Menu()->AddItem(tMenuItem);
				ModeNumber++;
		}
	}

	ApplyButton =	new BButton(
		BRect(
			200.0f,
			169.0f,
			263.0f,
			193.0f ),
		"Apply",
		"Apply",
		new BMessage('Aply'),
		B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM,
		B_WILL_DRAW | B_NAVIGABLE );
	if (ApplyButton != NULL)
	{
		ApplyButton->SetEnabled(false);
		MainView->AddChild(ApplyButton);
		ApplyButton->ResizeToPreferred();
		ApplyButton->MoveTo(
			MainView->Bounds().right - ApplyButton->Frame().Width() - 11.0f,
			MainView->Bounds().bottom - ApplyButton->Frame().Height() - 11.0f );
	}

}


clsMainWindow::~clsMainWindow()
{
}

void clsMainWindow::MessageReceived(BMessage * Message)
{
	switch(Message->what & 0xFFFFFF00 | 0x00000041)
	{
		case 'DevA':
			SelectedDevice = (Message->what - 'DevA');
			ApplyButton->SetEnabled((SelectedDevice >= 0) && (SelectedMode >= 0));
			break;
		case 'ModA':
			SelectedMode = (Message->what - 'ModA');
			ApplyButton->SetEnabled((SelectedDevice >= 0) && (SelectedMode >= 0));
			break;
		case 'AplA':
			if (Message->what == 'Aply')
			{
				DeviceManager * tDeviceManager(new DeviceManager);
				if (tDeviceManager != NULL) {
					GraphicsDevice * tGraphicsDevice(NULL);
					tGraphicsDevice = tDeviceManager->GetGraphicsDevice(SelectedDevice);
					if (tGraphicsDevice != NULL) {
						tGraphicsDevice->CreateBufferGroup();
						tGraphicsDevice->SetTVOutMode(SelectedMode);
						delete tGraphicsDevice;
						tGraphicsDevice = NULL;
					}
					delete tDeviceManager;
					tDeviceManager = NULL;
				}
			}
			break;
		default:
		  BWindow::MessageReceived(Message);
		  break;
	}
}

bool clsMainWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return BWindow::QuitRequested();
}
